//**************************************************************************************************
//
// TimeUtils.cpp: implementation of the CTimeUtils class.
//
// Description: Anything to do with time
//
// Created by: Keith Ditchburn 03/10/01
//
// Note: Singleton: call by using TIMER.
// Note: Requires project be linked to winmm.lib
//
//**************************************************************************************************

#define STRICT

#include <assert.h>
#include <stdio.h>
#include "windows.h"
#include "mmsystem.h"

#include "TimeUtils.h"

CTimeUtils *CTimeUtils::_instance=NULL;

// Can force this just to use timeGetTime() by uncommenting this:
//#define JUSTUSE_TIMEGETTIME

/**************************************************************************************************
**************************************************************************************************/
CTimeUtils::CTimeUtils()
{
	m_lerpValue=0.0f;
	m_lastTickTime=0.0f;
}

/**************************************************************************************************
**************************************************************************************************/
CTimeUtils::~CTimeUtils()
{
}

/**************************************************************************************************
Desc: Singleton handler
Date: 11/07/01
**************************************************************************************************/
CTimeUtils& CTimeUtils::Instance()
{
	if (_instance==NULL)
	{
		_instance = new CTimeUtils;
	}

	return *_instance;
}

/**************************************************************************************************
Desc: Singleton remover
Date: 11/07/01
**************************************************************************************************/
void CTimeUtils::RemoveInstance()
{
	if (_instance)
	{
		delete _instance;
		_instance=NULL;
	}
}

/**************************************************************************************************
Desc: See if chip supports querying performance and set up values
Date: 03/10/01
**************************************************************************************************/
void CTimeUtils::Initialise()
{
#ifdef JUSTUSE_TIMEGETTIME
		m_mmt_start= timeGetTime();
		m_tps= 1000;
		m_spt= 1.0f/1000.0f;
		m_pert_enable= false;

		DebugString("CTimeUtils: FORCED to use TGT\n");		
#else

	if (!QueryPerformanceFrequency((LARGE_INTEGER *)&m_tps)) 
	{ 
		m_mmt_start= timeGetTime();
		m_tps= 1000;
		m_spt= 1.0f/1000.0f;
		m_pert_enable= false;

		DebugString("CTimeUtils: QPF not supported, using TGT\n");
	}
	else
	{ 
		QueryPerformanceCounter((LARGE_INTEGER *)&m_pert_start); 
		m_spt= ((double)1.0)/((double)m_tps);
		m_pert_enable= true;

		DebugString("CTimeUtils: QPF supported, clock ticks per second = %d\n",m_tps);
	}
#endif
}

/**************************************************************************************************
Desc: Returns time in seconds, using best method supported
Date: 03/10/01
**************************************************************************************************/
float CTimeUtils::GetTimeInSeconds()
{
#ifdef JUSTUSE_TIMEGETTIME
	return(float)(((double)(timeGetTime()-m_mmt_start)) * m_spt);
#else

	__int64 time;

	if (m_pert_enable)
	{
		QueryPerformanceCounter((LARGE_INTEGER *)&time);
		return(float)(((double)(time-m_pert_start)) * m_spt);
	}
	else
	{
		return(float)(((double)(timeGetTime()-m_mmt_start)) * m_spt);
	}
#endif
}

/**************************************************************************************************
Desc: This is called each update, check for tick and calc lerp value
Returns true if it is time to update the world
Date: 03/01/2002
**************************************************************************************************/
bool CTimeUtils::FrameMove(float timeOfOneTick)
{
	bool timeToUpdate=FALSE;

	float timeNow=GetTimeInSeconds();

	if (timeNow - m_lastTickTime >= timeOfOneTick)
	{
		timeToUpdate=TRUE;
		m_lastTickTime=timeNow;
	}

	// Calc lerp value for objects - linear interpolation
	m_lerpValue=(timeNow - m_lastTickTime) / timeOfOneTick;

	// clamp - when paused etc. this may go above 1.0f
	if (m_lerpValue>1.0f)
		m_lerpValue=1.0f;

	return timeToUpdate;
}

/**************************************************************************************************
Desc: Return a lerp value based on current time and time since last tick
Date: 03/01/2002
**************************************************************************************************/
float CTimeUtils::GetLerpValue()
{
	return m_lerpValue;
}


/**************************************************************************************************
Desc: Allows a pause of the timer
Date: 15/08/2002
**************************************************************************************************/
void CTimeUtils::Pause(float numSeconds)
{
	float startTime=GetTimeInSeconds();
	float timeNow=startTime;

	while(timeNow<startTime+numSeconds)
		timeNow=GetTimeInSeconds();
}

/**************************************************************************************************
Desc: Displays a debug string to the output pane in viz, takes formatted text
Date: 25/04/2003
**************************************************************************************************/
void CTimeUtils::DebugString(char *text, ...)
{
	char tmp[2048];
 
	assert(text);

	if (strlen(text)>=2048)
	{
		OutputDebugString("MESSAGE TOO BIG\n");
		return;
	}
		
	if (strlen(text)==0) 
		return;

	va_list ptr ;

	va_start(ptr,text) ;

	vsprintf(tmp,text,ptr) ;

	OutputDebugString(tmp);
}